#include "utility/w5100.h"
#include "utility/socket.h"
extern "C" {
#include "string.h"
}

#include "Ethernet.h"
#include "EthernetClient.h"
#include "EthernetServer.h"

EthernetServer::EthernetServer(uint16_t port)
  : last_sock(MAX_SOCK_NUM)
  , max_socks(1)
  , socks_num(0)
{
  _port = port;
}

void EthernetServer::begin()
{
  for (int sock = 0; sock < MAX_SOCK_NUM; sock++) {
    EthernetClient client(sock);
    if (client.status() == SnSR::CLOSED) {
      socket(sock, SnMR::TCP, _port, 0);
      listen(sock);
      EthernetClass::_server_port[sock] = _port;
      if(++socks_num >= max_socks) {
        break;
      }
    }
  }
}

void EthernetServer::accept()
{
  int listening = 0;

  for (int sock = 0; sock < MAX_SOCK_NUM; sock++) {
    EthernetClient client(sock);

    if (EthernetClass::_server_port[sock] == _port) {
      if (client.status() == SnSR::LISTEN) {
        listening++;
      }
      else if (client.status() == SnSR::CLOSE_WAIT && !client.available()) {
        client.stop();
      }
    }
  }

  socks_num = listening;
  if (listening < max_socks) {
    begin();
  }
}

EthernetClient EthernetServer::available()
{
  accept();

  int sock = last_sock;
  for (int i = 0; i < MAX_SOCK_NUM; i++) {
    if (++sock >= MAX_SOCK_NUM)
      sock = 0;

    EthernetClient client(sock);
    if (EthernetClass::_server_port[sock] == _port &&
        (client.status() == SnSR::ESTABLISHED ||
         client.status() == SnSR::CLOSE_WAIT)) {
      if (client.available()) {
        last_sock = sock;
        return client;
      }
    }
  }

  return EthernetClient(MAX_SOCK_NUM);
}

void EthernetServer::setMaxSocks(int socks)
{
  max_socks = constrain(socks, 1, MAX_SOCK_NUM);
}

size_t EthernetServer::write(uint8_t b)
{
  return write(&b, 1);
}

size_t EthernetServer::write(const uint8_t *buffer, size_t size)
{
  size_t n = 0;

  accept();

  for (int sock = 0; sock < MAX_SOCK_NUM; sock++) {
    EthernetClient client(sock);

    if (EthernetClass::_server_port[sock] == _port &&
        client.status() == SnSR::ESTABLISHED) {
      n += client.write(buffer, size);
    }
  }

  return n;
}
